#! /usr/bin/env python

from __future__ import print_function
import sys
import openturns as ot

ot.TESTPREAMBLE()


def progress(percent):
    sys.stderr.write('-- progress=' + str(percent) + '%\n')


def stop():
    sys.stderr.write('-- stop?\n')
    return False


# We create a numerical math function
myFunction = ot.SymbolicFunction(['E', 'F', 'L', 'I'], ['-F*L^3/(3*E*I)'])

dim = myFunction.getInputDimension()

# We create a normal distribution point of dimension 1
mean = [0.0] * dim
# E
mean[0] = 50.0
# F
mean[1] = 1.0
# L
mean[2] = 10.0
# I
mean[3] = 5.0
sigma = [1.0] * dim
R = ot.IdentityMatrix(dim)
myDistribution = ot.Normal(mean, sigma, R)

# We create a 'usual' RandomVector from the Distribution
vect = ot.RandomVector(myDistribution)

# We create a composite random vector
output = ot.CompositeRandomVector(myFunction, vect)

# We create an Event from this RandomVector
myEvent = ot.ThresholdEvent(output, ot.Less(), -3.0)

# Monte Carlo
experiments = [ot.MonteCarloExperiment()]
# Quasi Monte Carlo
experiments.append(ot.LowDiscrepancyExperiment())
# Randomized Quasi Monte Carlo
experiment = ot.LowDiscrepancyExperiment()
experiment.setRandomize(True)
experiments.append(experiment)
# Importance sampling
mean[0] = 4.99689645939288809018e+01
mean[1] = 1.84194175946153282375e+00
mean[2] = 1.04454036676956398821e+01
mean[3] = 4.66776215562709406726e+00
myImportance = ot.Normal(mean, sigma, R)
experiments.append(ot.ImportanceSamplingExperiment(myImportance))
# Randomized LHS
experiment = ot.LHSExperiment()
experiment.setAlwaysShuffle(True)
experiments.append(experiment)

for experiment in experiments:

    ot.RandomGenerator.SetSeed(0)

    myAlgo = ot.ProbabilitySimulationAlgorithm(myEvent, experiment)
    myAlgo.setMaximumOuterSampling(250)
    myAlgo.setBlockSize(4)
    myAlgo.setMaximumCoefficientOfVariation(0.1)

    print('algo=', myAlgo)

    # Perform the simulation
    myAlgo.run()

    # Stream out the result
    print('algo result=', myAlgo.getResult())
    print('probability distribution=',
          myAlgo.getResult().getProbabilityDistribution())

    # Use the standard deviation as a stoping rule
    experiment = ot.MonteCarloExperiment()
    myAlgo = ot.ProbabilitySimulationAlgorithm(myEvent, experiment)
    myAlgo.setMaximumOuterSampling(250)
    myAlgo.setBlockSize(4)
    myAlgo.setMaximumCoefficientOfVariation(0.0)
    myAlgo.setMaximumStandardDeviation(0.1)
    myAlgo.setProgressCallback(progress)
    myAlgo.setStopCallback(stop)

    print('algo=', myAlgo)

    # Perform the simulation
    myAlgo.run()

    # Stream out the result
    print('algo result=', myAlgo.getResult())
    print('probability distribution=',
          myAlgo.getResult().getProbabilityDistribution())

print('-' * 32)
ot.RandomGenerator.SetSeed(0)
description = ot.Description()
description.add('composite vector/comparison event')
dim = 2
distribution = ot.Normal(dim)
Xvector = ot.RandomVector(distribution)
f = ot.SymbolicFunction(['x0', 'x1'], ['x0+x1'])
Yvector = ot.CompositeRandomVector(f, Xvector)
s = 1.0
event1 = ot.ThresholdEvent(Yvector, ot.Greater(), s)
description.add('composite vector/domain event')
domain1D = ot.LevelSet(ot.SymbolicFunction(
    ['x0'], ['sin(x0)']), ot.LessOrEqual(), -0.5)
event2 = ot.DomainEvent(Yvector, domain1D)
description.add('composite vector/interval event')
interval = ot.Interval(0.5, 1.5)
event3 = ot.ThresholdEvent(Yvector, interval)
description.add('process/domain event')
Xprocess = ot.WhiteNoise(distribution, ot.RegularGrid(0.0, 0.1, 10))
domain2D = ot.LevelSet(
    ot.SymbolicFunction(['x0', 'x1'], ['(x0-1)^2+x1^2']), ot.LessOrEqual(), 1.0)
event4 = ot.ProcessEvent(Xprocess, domain2D)
all_events = [event1, event2, event3, event4]
for i, event in enumerate(all_events):
    print(description[i])
    if event.isComposite():
        experiment = ot.MonteCarloExperiment()
        myAlgo = ot.ProbabilitySimulationAlgorithm(event, experiment)
    else:
        myAlgo = ot.ProbabilitySimulationAlgorithm(event)
    myAlgo.setMaximumOuterSampling(250)
    myAlgo.setBlockSize(4)
    myAlgo.setMaximumCoefficientOfVariation(0.1)
    myAlgo.run()
    print('MonteCarlo result=', myAlgo.getResult())
    print('probability distribution=',
          myAlgo.getResult().getProbabilityDistribution())



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

inputDimension = 3
outputDimension = 2
# Center
center = ot.Point(inputDimension)
center[0] = -1.0
center[1] = 0.5
center[2] = 1.0
# Constant term
constant = ot.Point(outputDimension)
constant[0] = -1.0
constant[1] = 2.0
# Linear term
linear = ot.Matrix(inputDimension, outputDimension)
linear[0, 0] = 1.0
linear[1, 0] = 2.0
linear[2, 0] = 3.0
linear[0, 1] = 4.0
linear[1, 1] = 5.0
linear[2, 1] = 6.0
# Quadratic term
quadratic = ot.SymmetricTensor(inputDimension, outputDimension)
quadratic[0, 0, 0] = 7.0
quadratic[0, 0, 1] = -7.0
quadratic[0, 1, 0] = 8.0
quadratic[0, 1, 1] = -8.0
quadratic[0, 2, 0] = 9.0
quadratic[0, 2, 1] = -9.0
quadratic[1, 0, 0] = 8.0
quadratic[1, 0, 1] = -8.0
quadratic[1, 1, 0] = 10.0
quadratic[1, 1, 1] = -10.0
quadratic[1, 2, 0] = 11.0
quadratic[1, 2, 1] = -11.0
quadratic[2, 0, 0] = 9.0
quadratic[2, 0, 1] = -9.0
quadratic[2, 1, 0] = 11.0
quadratic[2, 1, 1] = -11.0
quadratic[2, 2, 0] = 12.0
quadratic[2, 2, 1] = -12.0

# myFunction = linear * (X- center) + constant
myFunction = ot.QuadraticFunction(center, constant, linear, quadratic)
inPoint = ot.Point(inputDimension)
inPoint[0] = 7.0
inPoint[1] = 8.0
inPoint[2] = 9.0
outPoint = myFunction(inPoint)
print('myFunction=', repr(myFunction))
print(myFunction.getName(), '( ', repr(inPoint), ' ) = ', repr(outPoint))
print(myFunction.getName(), '.gradient( ', repr(inPoint), ' ) = ',
      repr(myFunction.gradient(inPoint)))
print(myFunction.getName(), '.hessian( ', repr(inPoint), ' ) = ',
      repr(myFunction.hessian(inPoint)))



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    # Dimension of the input model
    # Size of the TimeGrid
    size = 64
    dimension = 1
    timeGrid = RegularGrid(0.0, 0.1, size)
    amplitude = Point(dimension, 2.0)
    scale = Point(dimension, 1.0)
    model = CauchyModel(scale, amplitude)
    myProcess = SpectralGaussianProcess(model, timeGrid)

    # Create a Process sample
    N = 100
    sample = ProcessSample(myProcess.getSample(N))

    # Filtering Windows
    myFactory = WelchFactory()

    # Build a UserDefinedSpectralModel using the Wellch method
    mySpectralModel = myFactory.buildAsUserDefinedSpectralModel(sample)

    # Get the frequency grid of the model
    myFrequencyGrid = mySpectralModel.getFrequencyGrid()
    for i in range(dimension):
        for j in range(dimension):
            print("Spectre ", i, "-", j)
            for k in range(myFrequencyGrid.getN()):
                frequency = myFrequencyGrid.getStart(
                ) + k * myFrequencyGrid.getStep()
                estimatedValue = (
                    mySpectralModel(frequency)[i, j]).real
                modelValue = (
                    model(frequency)[i, j]).real
                print("Frequency =  %.6f" % frequency, ", evaluation = %.8f" %
                      estimatedValue, " model = %.8f" % modelValue)

    # Create a Time Series
    timeSeries = myProcess.getRealization()

    # Build a UserDefinedSpectralModel using the Wellch method
    mySpectralModel2 = myFactory.buildAsUserDefinedSpectralModel(timeSeries)

    # Get the frequency grid of the model
    myFrequencyGrid = mySpectralModel2.getFrequencyGrid()
    for i in range(dimension):
        for j in range(dimension):
            print("Spectre ", i, "-", j)
            for k in range(myFrequencyGrid.getN()):
                frequency = myFrequencyGrid.getStart(
                ) + k * myFrequencyGrid.getStep()
                estimatedValue = (
                    mySpectralModel2(frequency)[i, j]).real
                modelValue = (
                    model(frequency)[i, j]).real
                print("Frequency =  %.6f" % frequency, ", evaluation = %.8f" %
                      estimatedValue, " model = %.8f" % modelValue)

except:
    import sys
    print("t_WelchFactory_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

levels = [4] * 2
experiment = ot.Box(levels)
print('experiment = ', experiment)
sample = experiment.generate()
print('sample = ', repr(sample))

experiment = ot.Box(levels, ot.Interval([7.0, 8.0], [9.0, 16.0]))
print('experiment = ', experiment)
sample = experiment.generate()
print('sample = ', repr(sample))



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    distribution = PlackettCopula(1.5)
    size = 1000
    sample = distribution.getSample(size)
    factory = PlackettCopulaFactory()
    estimatedDistribution = factory.build(sample)
    print("distribution=", repr(distribution))
    print("Estimated distribution=", repr(estimatedDistribution))
    estimatedPlackettCopula = factory.buildAsPlackettCopula(sample)
    print("PlackettCopula          =", distribution)
    print("Estimated plackettCopula=", estimatedPlackettCopula)
    estimatedPlackettCopula = factory.buildAsPlackettCopula()
    print("Default plackettCopula=", estimatedPlackettCopula)
    estimatedPlackettCopula = factory.buildAsPlackettCopula(
        distribution.getParameter())
    print("PlackettCopula from parameters=", estimatedPlackettCopula)

except:
    import sys
    print("t_PlackettFactory_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    # We create an empty Sample
    N = 100
    sample = Sample(N, 2)
    print("Sample size=", sample.getSize())

    # We populate the empty sample with numerous data
    for i in range(N):
        point = Point(2)
        point[0] = 1000 + i
        point[1] = 2000 + i
        sample[i] = point
    print("sample=", repr(sample))

    # We try to split the sample
    sample90 = sample.split(90)
    sample10 = sample.split(10)
    print("sample  =", repr(sample))
    print("sample10=", repr(sample10))
    print("sample90=", repr(sample90))

# except TestFailed, ex :
except:
    import sys
    print("t_Sample_split.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:

    # We create a numerical math function
    myFunction = SymbolicFunction(
        ('E', 'F', 'L', 'I'), ('-F*L^3/(3.*E*I)',))

    dim = myFunction.getInputDimension()
    # We create a normal distribution point of dimension 1
    mean = Point(dim, 0.0)
    # E
    mean[0] = 50.0
    # F
    mean[1] = 1.0
    # L
    mean[2] = 10.0
    # I
    mean[3] = 5.0
    sigma = Point(dim, 1.0)
    R = IdentityMatrix(dim)
    myDistribution = Normal(mean, sigma, R)

    # We create a 'usual' RandomVector from the Distribution
    vect = RandomVector(myDistribution)

    # We create a composite random vector */
    output = CompositeRandomVector(myFunction, vect)

    # We create an Event from this RandomVector */
    myEvent = ThresholdEvent(output, Less(), -3)

# TEST ONE : directional sampling from an event (slow and safe strategy by default)
    # We create a Directional Sampling algorithm */
    myAlgo = DirectionalSampling(myEvent)
    myAlgo.setMaximumOuterSampling(250)
    myAlgo.setBlockSize(4)
    myAlgo.setMaximumCoefficientOfVariation(0.1)

    print("DirectionalSampling=", myAlgo)

    # Perform the simulation */
    myAlgo.run()

    # Stream out the result */
    print("DirectionalSampling result=", myAlgo.getResult())

# TEST TWO : diectional sampling from an event and a root strategy : Risky And Fast, and a directional sampling strategy :
    # We create a Directional Sampling algorithm */
    myAlgo2 = DirectionalSampling(myEvent, MediumSafe(), OrthogonalDirection())
    myAlgo2.setMaximumOuterSampling(250)
    myAlgo2.setBlockSize(4)
    myAlgo2.setMaximumCoefficientOfVariation(0.1)

    print("DirectionalSampling=", myAlgo2)

    # Perform the simulation */
    myAlgo2.run()

    # Stream out the result */
    print("DirectionalSampling result=", myAlgo2.getResult())

    # Ticket #778
    myFunction = SymbolicFunction('x', 'x')

    X1 = RandomVector(Uniform(-2, 1.99999))
    X2 = RandomVector(Uniform(-2, 2.0))

    for X in [X1, X2]:
        vector = CompositeRandomVector(myFunction, X)
        event = ThresholdEvent(vector, GreaterOrEqual(), 0.0)

        print('X:', X.getDistribution())

        myAlgo3 = DirectionalSampling(event)
        n1 = myFunction.getCallsNumber()
        myAlgo3.run()
        n2 = myFunction.getCallsNumber()
        result = myAlgo3.getResult().getProbabilityEstimate()
        print('p=%.6g (ncalls = %d)' % (result, n2 - n1))

except:
    import sys
    print("t_DirectionalSampling.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    mesh = IntervalMesher([9]).build(Interval(-1.0, 1.0))
    cov1D = AbsoluteExponential([1.0])
    algo = KarhunenLoeveP1Algorithm(mesh, cov1D, 0.0)
    algo.run()
    result = algo.getResult()
    projection = KarhunenLoeveProjection(result)
    print("projection=", projection)
    # Get the input and output description
    print("projection input description=", projection.getInputDescription())
    print("projection output description=", projection.getOutputDescription())
    # Get the spatial, input and output dimension
    print("projection input dimension=",
          projection.getInputMesh().getDimension())
    print("projection input dimension=", projection.getInputDimension())
    print("projection output dimension=", projection.getOutputDimension())
    field = result.getModesAsProcessSample().computeMean()
    point = projection(field)
    print("field=", field)
    print("point=", point)
    # Get the number of calls
    print("called ", projection.getCallsNumber(), " times")

except:
    import sys
    print("t_KarhunenLoeveProjection_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    size = 100
    xuniform = Uniform(0.9, 1.1)
    x = xuniform.getSample(size)
    yuniform = Uniform(1.9, 2.1)
    y = yuniform.getSample(size)
    w = [1.0] * size
    f = SymbolicFunction(['x'], ['2.0*x'])
    basis = [f]
    indices = list(range(len(basis)))

    fittingAlgo = KFold()

    print("algo =", fittingAlgo)

    result = fittingAlgo.run(x, y, w, basis, indices)

    print("result = %g" % result)


except:
    import sys
    print("t_KFold_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    dim = 3

    copula = MinCopula(dim)
    copula.setName("a min copula")
    print("Copula ", repr(copula))
    print("Copula ", copula)
    print("Mean ", copula.getMean())
    print("Covariance ", copula.getCovariance())
    # Is this copula an elliptical distribution?
    print("Elliptical distribution= ", copula.isElliptical())

    # Is this copula elliptical ?
    print("Elliptical copula= ", copula.hasEllipticalCopula())

    # Is this copula independent ?
    print("Independent copula= ", copula.hasIndependentCopula())

    # Test for realization of copula
    oneRealization = copula.getRealization()
    print("oneRealization=", oneRealization)

    # Test for sampling
    size = 10
    oneSample = copula.getSample(size)
    print("oneSample=", oneSample)

    # Test for sampling
    size = 10000
    anotherSample = copula.getSample(size)
    print("anotherSample mean=", anotherSample.computeMean())
    print("anotherSample covariance=", anotherSample.computeCovariance())

    # Define a point
    point = Point(dim, 0.2)

    # Show PDF and CDF of zero point
    zeroPDF = copula.computePDF(point)
    zeroCDF = copula.computeCDF(point)
    print("point= ", point, "pdf=", zeroPDF, "cdf=", zeroCDF)

    # Get 50% quantile
    quantile = copula.computeQuantile(0.5)
    print("Quantile=", quantile)
    print("CDF(quantile)=", copula.computeCDF(quantile))

    # Get 5% quantile
    quantile = copula.computeQuantile(0.95, True)
    print("Quantile=", quantile)

    # Get 95% survival function
    inverseSurvival = Point(copula.computeInverseSurvivalFunction(0.95))
    print("InverseSurvival=", repr(inverseSurvival))
    print("Survival(inverseSurvival)=%.6f" %
          copula.computeSurvivalFunction(inverseSurvival))
    print("entropy=%.6e" % copula.computeEntropy())
    # Extract the marginals
    for i in range(dim):
        margin = copula.getMarginal(i)
        print("margin=", margin)
        print("margin PDF=", margin.computePDF(Point(1, 0.25)))
        print("margin CDF=", margin.computeCDF(Point(1, 0.25)))
        print("margin quantile=", margin.computeQuantile(0.95))
        print("margin realization=", margin.getRealization())

    # Extract a 2-D marginal
    indices = Indices(2, 0)
    indices[0] = 1
    indices[1] = 0
    print("indices=", indices)
    margins = copula.getMarginal(indices)
    print("margins=", margins)
    print("margins PDF=", margins.computePDF(Point(2, 0.25)))
    print("margins CDF=", margins.computeCDF(Point(2, 0.25)))
    quantile = margins.computeQuantile(0.95)
    print("margins quantile=", quantile)
    print("margins CDF(quantile)=", margins.computeCDF(quantile))
    print("margins realization=", margins.getRealization())

except:
    import sys
    print("t_MinCopula.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()

mesh = ot.RegularGrid(0.0, 0.1, 11)


def f(X):
    size = 11
    Y = [ot.Point(X)*i for i in range(size)]
    return Y


inputDim = 2
outputDim = 2
function = ot.PythonPointToFieldFunction(inputDim, mesh, outputDim, f)
function.setInputDescription(['u1', 'u2'])
function.setOutputDescription(['v1', 'v2'])

# freeze u1=5.0
parametric = ot.ParametricPointToFieldFunction(function, [0], [5.0])

# properties
print ('dim=', parametric.getInputDimension(), parametric.getOutputDimension())
print ('description=', parametric.getInputDescription(),
       parametric.getOutputDescription())
print ('mesh=', parametric.getOutputMesh())

# op(Point)
x = [1.0]
y = parametric(x)
print('y=', y)
print ('calls=', parametric.getCallsNumber())


# op(Sample)
X = [[1.0], [7.0]]
Y = parametric(X)
print('Y=', Y)
print ('calls=', parametric.getCallsNumber())



#! /usr/bin/env python

from __future__ import print_function
from openturns import *
import math

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    f = SymbolicFunction(["x"], ["sin(x)"])
    a = -2.5
    b = 4.5
    # Default parameters
    algo = GaussLegendre()
    print("Algo=", algo)
    # High-level interface
    value, adaptedNodes = algo.integrateWithNodes(f, Interval(a, b))
    ref = math.cos(a) - math.cos(b)
    print("value=%.6f" %
          value[0], ", ref=%.6f" % ref, ", adaptedNodes=", adaptedNodes)
    # Low-level interface
    algo = GaussLegendre([20])
    value, adaptedNodes = algo.integrateWithNodes(f, Interval(a, b))
    print("value=%.6f" %
          value[0], ", ref=%.6f" % ref, ", adaptedNodes=", adaptedNodes)

except:
    import sys
    print("t_GaussKronrod_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

# We create a distribution
distribution = ot.Normal()

print('distribution = ', repr(distribution))

aCollection = []
aCollection.append(ot.Normal(0.0, 1.0))
aCollection.append(ot.Uniform(1.0, 1.5))
distributionParameters = ot.ComposedDistribution(aCollection)
randomParameters = ot.RandomVector(distributionParameters)

print('random parameters=', randomParameters)

# We create a distribution-based conditional RandomVector
vect = ot.ConditionalRandomVector(distribution, randomParameters)
print('vect=', vect)

# Check standard methods of class RandomVector
print('vect dimension=', vect.getDimension())
p = ot.Point()
r = vect.getRealization(p)
print('vect realization=', repr(r))
print('parameters value=', repr(p))
distribution.setParameter(p)
ot.RandomGenerator.SetSeed(0)
# Generate a parameter set to put the random generator into the proper
# state
randomParameters.getRealization()
# The realization of the distribution should be equal to the realization
# of the conditional vector
print('dist realization=', repr(distribution.getRealization()))

print('vect sample =', repr(vect.getSample(5)))

parameter = vect.getParameter()
print("parameter =", repr(parameter))
vect.setParameter(parameter)
print("parameter desc =", repr(vect.getParameterDescription()))



#! /usr/bin/env python

from __future__ import print_function
from openturns import *
from math import *

TESTPREAMBLE()

try:
    # Create an intance
    inputVars = Description(["t"])
    formula = Description(["sin(t)", "cos(t)"])
    myFunc = SymbolicFunction(inputVars, formula)
    tg = RegularGrid(0.0, 0.1, 11)
    myInverseTrendFunc = InverseTrendTransform(myFunc, tg)

    print("myInverseTrendFunc=", myInverseTrendFunc)
    # Get the input description and dimension
    print("myInverseTrendFunc input description=",
          myInverseTrendFunc.getInputDescription())
    print("myInverseTrendFunc input dimension=",
          myInverseTrendFunc.getInputDimension())
    # Get the output description and dimension
    print("myInverseTrendFunc output description=",
          myInverseTrendFunc.getOutputDescription())
    print("myInverseTrendFunc output dimension=",
          myInverseTrendFunc.getOutputDimension())
    # Create a TimeSeries
    data = Sample(tg.getN(), formula.getSize())
    for i in range(data.getSize()):
        t = tg.getStart() + i * tg.getStep()
        data[i, 0] = i + 1 + sin(t)
        data[i, 1] = i + cos(t)
    ts = TimeSeries(tg, data)
    print("input time series =  ")
    print(ts)
    print("output time series = ")
    print(myInverseTrendFunc(ts))
    # Get the number of calls
    print("called ", myInverseTrendFunc.getCallsNumber(), " times")

except:
    import sys
    print("t_InverseTrendTransform_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()

try:
    l = [1.0, 0.7, 1.2, 0.9]
    h = [0.5, 1.5, 3.5, 2.5]
    distribution = ot.Histogram(-1.5, l, h)
    size = 10000
    sample = distribution.getSample(size)
    factory = ot.HistogramFactory()
    estimatedDistribution = factory.build(sample)
    print("distribution=", repr(distribution))
    print("Estimated distribution=", repr(estimatedDistribution))
    estimatedDistribution = factory.build()
    print("Default distribution=", estimatedDistribution)
    estimatedHistogram = factory.buildAsHistogram(sample)
    print("Histogram          =", distribution)
    print("Estimated histogram=", estimatedHistogram)
    estimatedHistogram = factory.buildAsHistogram(sample, 0.1)
    print("Histogram          =", distribution)
    print("Estimated histogram=", estimatedHistogram)
    estimatedHistogram = factory.buildAsHistogram(sample, 15)
    print("Histogram          =", distribution)
    print("Estimated histogram=", estimatedHistogram)
    first = -2.
    width = ot.Point(5, 1.)
    estimatedHistogram = factory.buildAsHistogram(sample, first, width)
    print("Estimated histogram=", estimatedHistogram)
    estimatedHistogram = factory.buildAsHistogram()
    print("Default histogram=", estimatedHistogram)
    sample = [[0.0]] * size
    estimatedDistribution = factory.build(sample)
    print("Estimated distribution=", repr(estimatedDistribution))
    sample = [[1.0]] * size
    estimatedDistribution = factory.build(sample)
    print("Estimated distribution=", repr(estimatedDistribution))

    sample = ot.ChiSquare(0.0120637).getSample(32)
    for useQuantile in [True, False]:
        print('silverman useQuantile=', useQuantile, ot.Point(
            1, ot.HistogramFactory().computeSilvermanBandwidth(sample, useQuantile)))
except:
    import sys
    print("t_HistogramFactory_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    # Instanciate one distribution object
    dim = 3
    R = CorrelationMatrix(dim)
    for i in range(dim - 1):
        R[i, i + 1] = 0.25
    copula = SklarCopula(
        Distribution(Normal([1.0, 2.0, 3.0], [2.0, 3.0, 1.0], R)))
    copulaRef = NormalCopula(R)
    print("Copula ", repr(copula))
    print("Copula ", copula)
    print("Mean      =", repr(copula.getMean()))
    print("Mean (ref)=", repr(copulaRef.getMean()))
    ResourceMap.SetAsUnsignedInteger("GaussKronrod-MaximumSubIntervals", 20)
    ResourceMap.SetAsScalar("GaussKronrod-MaximumError",  1.0e-4)
    print("Covariance      =", repr(copula.getCovariance()))
    ResourceMap.SetAsUnsignedInteger("GaussKronrod-MaximumSubIntervals", 100)
    ResourceMap.SetAsScalar("GaussKronrod-MaximumError",  1.0e-12)
    print("Covariance (ref)=", repr(copulaRef.getCovariance()))

    # Is this copula an elliptical distribution?
    print("Elliptical distribution= ", copula.isElliptical())

    # Is this copula elliptical ?
    print("Elliptical copula= ", copula.hasEllipticalCopula())

    # Is this copula independent ?
    print("Independent copula= ", copula.hasIndependentCopula())

    # Test for realization of distribution
    oneRealization = copula.getRealization()
    print("oneRealization=", repr(oneRealization))

    # Test for sampling
    size = 10
    oneSample = copula.getSample(size)
    print("oneSample=", repr(oneSample))

   # Test for sampling
    size = 1000
    anotherSample = copula.getSample(size)
    print("anotherSample mean=", repr(anotherSample.computeMean()))
    print("anotherSample covariance=", repr(anotherSample.computeCovariance()))

    # Define a point
    point = Point(dim, 0.2)

    # Show PDF and CDF of point
    pointPDF = copula.computePDF(point)
    pointCDF = copula.computeCDF(point)
    pointPDFRef = copulaRef.computePDF(point)
    pointCDFRef = copulaRef.computeCDF(point)
    print("Point = ", repr(point), " pdf      =%.6f" %
          pointPDF, " cdf      =%.6f" % pointCDF)
    print("Point = ", repr(point), " pdf (ref)=%.6f" %
          pointPDFRef, " cdf (ref)=%.6f" % pointCDFRef)

    # Get 50% quantile
    quantile = copula.computeQuantile(0.5)
    quantileRef = copulaRef.computeQuantile(0.5)
    print("Quantile      =", repr(quantile))
    print("Quantile (ref)=", repr(quantileRef))
    print("CDF(quantile)=%.6f" % copula.computeCDF(quantile))
    # Get 95% survival function
    inverseSurvival = Point(copula.computeInverseSurvivalFunction(0.95))
    print("InverseSurvival=", repr(inverseSurvival))
    print("Survival(inverseSurvival)=%.6f" %
          copula.computeSurvivalFunction(inverseSurvival))
    # Takes too much time
    # print("entropy=%.6f" % copula.computeEntropy())
    # Confidence regions
    if copula.getDimension() <= 2:
        threshold = Point()
        print("Minimum volume interval=",
              copula.computeMinimumVolumeInterval(0.95, threshold))
        print("threshold=", threshold)
        beta = Point()
        levelSet = copula.computeMinimumVolumeLevelSet(0.95, beta)
        print("Minimum volume level set=", levelSet)
        print("beta=", beta)
        print("Bilateral confidence interval=",
              copula.computeBilateralConfidenceInterval(0.95, beta))
        print("beta=", beta)
        print("Unilateral confidence interval (lower tail)=",
              copula.computeUnilateralConfidenceInterval(0.95, False, beta))
        print("beta=", beta)
        print("Unilateral confidence interval (upper tail)=",
              copula.computeUnilateralConfidenceInterval(0.95, True, beta))
        print("beta=", beta)

    # Extract the marginals
    for i in range(dim):
        margin = copula.getMarginal(i)
        marginRef = copulaRef.getMarginal(i)
        print("margin=", repr(margin))
        print("margin PDF      =%.6f" %
              margin.computePDF(Point(1, 0.25)))
        print("margin PDF (ref)=%.6f" %
              marginRef.computePDF(Point(1, 0.25)))
        print("margin CDF      =%.6f" %
              margin.computeCDF(Point(1, 0.25)))
        print("margin CDF (ref)=%.6f" %
              marginRef.computeCDF(Point(1, 0.25)))
        print("margin quantile      =", repr(margin.computeQuantile(0.95)))
        print("margin quantile (ref)=", repr(marginRef.computeQuantile(0.95)))
        print("margin realization=", repr(margin.getRealization()))

    # Extract a 2-D marginal
    indices = Indices(2, 0)
    indices[0] = 1
    indices[1] = 0
    print("indices=", repr(indices))
    margin = copula.getMarginal(indices)
    marginRef = copulaRef.getMarginal(indices)
    print("margin=", repr(margin))
    print("margin PDF      =%.6f" % margin.computePDF(Point(2, 0.25)))
    print("margin PDF (ref)=%.6f" %
          marginRef.computePDF(Point(2, 0.25)))
    print("margin CDF      =%.6f" % margin.computeCDF(Point(2, 0.25)))
    print("margin CDF (ref)=%.6f" %
          marginRef.computeCDF(Point(2, 0.25)))
    print("margin quantile      =", repr(margin.computeQuantile(0.95)))
    print("margin quantile (ref)=", repr(marginRef.computeQuantile(0.95)))
    print("margin realization=", repr(margin.getRealization()))

except:
    import sys
    print("t_SklarCopula.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *
from math import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:

    dim = 2
    transformation = NatafIndependentCopulaHessian(dim)
    print("transformation=", repr(transformation))
    point = Point(dim, 0.75)
    print("transformation(", point, ")=", repr(transformation.hessian(point)))
    print("input dimension=", transformation.getInputDimension())
    print("output dimension=", transformation.getOutputDimension())

except:
    import sys
    print("t_NatafIndependentCopulaHessian_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:

    # this analitycal example is taken from "Bayesian Modeling Using WinBUGS" - Ioannis Ntzoufras
    # 1.5.3: Inference for the mean or normal data with known variance

    # Variable of interest: Y=N(mu, sigma)
    # Prior for mu: Normal(mu0, sigma0), sigma is known
    # Posterior for mu: E(mu|y)=w*y_mean+(1-w)*mu0, and Var(mu|y)=w*(sigmay^2)/n
    # => weighted average of the prior an the sample mean
    # with w = n*sigma0^2 / (n*sigma0^2 + sigma^2)

    # Log::Show(Log::ALL)
    # observations
    size = 10
    realDist = Normal(31., 1.2)

    data = realDist.getSample(size)

    # calibration parameters
    calibrationColl = [CalibrationStrategy()] * 2

    # proposal distribution
    proposalColl = DistributionCollection()
    mean_proposal = Uniform(-2.0, 2.0)
    std_proposal = Uniform(-2.0, 2.0)
    proposalColl.add(mean_proposal)
    proposalColl.add(std_proposal)

    # prior distribution
    mu0 = 25.

    sigma0s = [0.1, 1.0]
    # sigma0s.append(2.0)

    # play with the variance of the prior:
    # if the prior variance is low (information concernig the mu parameter is strong)
    # then the posterior mean will be equal to the prior mean
    # if large, the the posterior distribution is equivalent to the
    # distribution of the sample mean
    for i in range(len(sigma0s)):

        sigma0 = sigma0s[i]
        mean_prior = Normal(mu0, sigma0)
        std_prior = Dirac(2.0)  # standard dev is known
        prior = ComposedDistribution([mean_prior, std_prior])

        # choose the initial state within the prior
        initialState = prior.getRealization()

        # conditional distribution
        conditional = Normal()

        # create a metropolis-hastings sampler
        sampler = RandomWalkMetropolisHastings(
            prior, conditional, data, initialState, proposalColl)
        sampler.setVerbose(True)
        sampler.setThinning(2)
        sampler.setBurnIn(500)
        sampler.setCalibrationStrategyPerComponent(calibrationColl)
        realization = sampler.getRealization()

        sigmay = ConditionalDistribution(
            Normal(), prior).getStandardDeviation()[0]
        w = size * sigma0 ** 2. / (size * sigma0 ** 2. + sigmay ** 2.0)

        print("prior variance= %.12g" % (sigma0 ** 2.))
        print("  realization=", realization)

        print("  w= %.12g" % w)

        # the posterior for mu is analytical
        print("  expected posterior ~N( %.12g" % (w * data.computeMean()
                                                  [0] + (1. - w) * mu0), ",  %.12g" % ((w * sigmay ** 2.0 / size) ** 0.5), ")")

        # try to generate a sample
        sample = sampler.getSample(50)

        print("  obtained posterior ~N( %.12g" % sample.computeMean()[
              0], ",  %.12g" % sample.computeStandardDeviationPerComponent()[0], ")")

        print("  acceptance rate=", sampler.getAcceptanceRate())


except:
    import sys
    import traceback
    traceback.print_exc(file=sys.stdout)



#! /usr/bin/env python

from __future__ import print_function
from openturns import *
from math import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    sample = Sample(4, 3)
    sample[0] = Point((1, 0, 9))
    sample[1] = Point((2, 3, 5))
    sample[2] = Point((5, 1, 8))
    sample[3] = Point((6, 7, 2))

    print("sample=", repr(sample))
    print("min=", repr(sample.getMin()))
    print("max=", repr(sample.getMax()))
    print("mean=", repr(sample.computeMean()))
    print("covariance=", repr(sample.computeCovariance()))
    print("standard deviation=", repr(sample.computeStandardDeviation()))
    print("standard deviation per component=", repr(
        sample.computeStandardDeviationPerComponent()))
    print("Pearson correlation=", repr(sample.computePearsonCorrelation()))
    print("Spearman correlation=", repr(sample.computeSpearmanCorrelation()))
    print("Kendall tau=", repr(sample.computeKendallTau()))
    print("range per component=", repr(sample.computeRange()))
    print("median per component=", repr(sample.computeMedian()))
    print("Variance=", repr(sample.computeVariance()))
    print("Skewness=", repr(sample.computeSkewness()))
    print("Kurtosis=", repr(sample.computeKurtosis()))
    for i in range(10):
        print("Centered moment of order ", i, "=",
              sample.computeCenteredMoment(i))
    print("Marginal 1=", repr(sample.getMarginal(1)))
    indices = Indices(2)
    indices[0] = 2
    indices[1] = 0
    print("Marginal [2, 0]=", repr(sample.getMarginal(indices)))
    print("Rank=", repr(sample.rank()))
    print("Sort=", repr(sample.sort()))
    print("Sort according to component 0=", repr(
        sample.sortAccordingToAComponent(0)))
    prob = 0.25
    print("Quantile per component(", prob, ")=", repr(
        sample.computeQuantilePerComponent(prob)))
    probs = [0.25, 0.75]
    print("Quantile per component(", probs, ")=", repr(
        sample.computeQuantilePerComponent(probs)))
    probs = [0.75, 0.25]
    print("Quantile per component(", probs, ")=", repr(
        sample.computeQuantilePerComponent(probs)))
    pointCDF = Point(sample.getDimension(), 0.25)
    print(
        "Empirical CDF(", repr(pointCDF), "=", sample.computeEmpiricalCDF(pointCDF))
    dim = 3
    R = CorrelationMatrix(dim)
    for i in range(1, dim):
        R[i, i - 1] = 0.25
    Rtmp = CorrelationMatrix(dim)
    for i in range(dim):
        for j in range(i):
            Rtmp[i, j] = 6.0 * asin(R[i, j] / 2.0) / pi
    print("Pearson correlation (exact)=", repr(R))
    print("Spearman correlation (exact)=", repr(Rtmp))
    sample[0] = Point((1, 0, 9))
    sample[1] = Point((2, 3, 9))
    sample[2] = Point((5, 1, 9))
    sample[3] = Point((2, 2, 9))
    print("Rank=", repr(sample.rank()))
    print("Rank component 0=", repr(sample.rank(0)))
    print("Sort           =", repr(sample.sort()))
    sample2 = Sample(sample)
    sample2.sortInPlace()
    print("Sort (in place)=", repr(sample2))
    print("Sort according to component 0           =", repr(
        sample.sortAccordingToAComponent(0)))
    sample2 = Sample(sample)
    sample2.sortAccordingToAComponentInPlace(0)
    print("Sort according to component 0 (in place)=", repr(sample2))
    print("Spearman correlation=", repr(sample.computeSpearmanCorrelation()))
    print("Kendall tau=", repr(sample.computeKendallTau()))
    size = 10000
    normal = Normal(Point(dim, 0.0), Point(dim, 1.0), R)
    print("Normal=", repr(normal))
    print("covariance=", repr(normal.getCovariance()))
    normalSample = normal.getSample(size)
    print("Empirical covariance=", repr(normalSample.computeCovariance()))
    RPearson = normalSample.computePearsonCorrelation()
    print("Pearson correlation=", repr(RPearson))
    RSpearman = normalSample.computeSpearmanCorrelation()
    print("Spearman correlation=", repr(RSpearman))
    print("Unique           =", repr(sample.sortUnique()))
    sample2 = Sample(sample)
    sample2.sortUniqueInPlace()
    print("Unique (in place)=", repr(sample2))
    sample[2] = Point((1, 0, 9))
    print("Unique           =", repr(sample.sortUnique()))
    sample2 = Sample(sample)
    sample2.sortUniqueInPlace()
    print("Unique (in place)=", repr(sample2))
# except TestFailed, ex :
except:
    import sys
    print("t_Sample_computation.py",
          sys.exc_info()[0], sys.exc_info()[1])
